#ifndef MSP_GEOMETRY_INTERSECTION_H_
#define MSP_GEOMETRY_INTERSECTION_H_

#include "compositeshape.h"

namespace Msp {
namespace Geometry {

template<typename T, unsigned D>
struct IntersectionOps
{
	static BoundingBox<T, D> combine_aabb(const BoundingBox<T, D> &a, const BoundingBox<T, D> &b) { return a&b; }
	static Coverage combine_coverage(Coverage a, Coverage b) { return ((a==PARTIAL_COVERAGE && b==a) ? UNCERTAIN_COVERAGE : std::min(a, b)); }
	static bool shortcircuit(bool c) { return !c; }
};

/**
Forms a shape from the common parts of component shapes.
*/
template<typename T, unsigned D>
class Intersection: public CompositeShape<T, D, IntersectionOps<T, D> >
{
private:
	Intersection() { }
public:
	Intersection(const Shape<T, D> &, const Shape<T, D> &);

	template<typename Iter>
	static Intersection from_iterator_range(const Iter &, const Iter &);

	virtual Intersection *clone() const;
};

template<typename T, unsigned D>
inline Intersection<T, D>::Intersection(const Shape<T, D> &s1, const Shape<T, D> &s2):
	CompositeShape<T, D, IntersectionOps<T, D> >(s1, s2)
{ }

template<typename T, unsigned D>
template<typename Iter>
inline Intersection<T, D> Intersection<T, D>::from_iterator_range(const Iter &begin, const Iter &end)
{
	Intersection<T, D> shape;
	shape.init_from_iter_range(begin, end);
	return shape;
}

template<typename T, unsigned D>
inline Intersection<T, D> *Intersection<T, D>::clone() const
{
	return new Intersection<T, D>(*this);
}

} // namespace Geometry
} // namespace Msp

#endif
