#ifndef MSP_GEOMETRY_UNION_H_
#define MSP_GEOMETRY_UNION_H_

#include "compositeshape.h"

namespace Msp {
namespace Geometry {

template<typename T, unsigned D>
struct UnionOps
{
	static BoundingBox<T, D> combine_aabb(const BoundingBox<T, D> &a, const BoundingBox<T, D> &b) { return a|b; }
	static Coverage combine_coverage(Coverage a, Coverage b) { return std::max(a, b); }
	static bool shortcircuit(bool c) { return c; }
};

/**
Joins component shapes together into one.
*/
template<typename T, unsigned D>
class Union: public CompositeShape<T, D, UnionOps<T, D> >
{
private:
	Union() { }
public:
	Union(const Shape<T, D> &, const Shape<T, D> &);

	template<typename Iter>
	static Union from_iterator_range(const Iter &, const Iter &);

	virtual Union *clone() const;
};

template<typename T, unsigned D>
inline Union<T, D>::Union(const Shape<T, D> &s1, const Shape<T, D> &s2):
	CompositeShape<T, D, UnionOps<T, D> >(s1, s2)
{ }

template<typename T, unsigned D>
template<typename Iter>
inline Union<T, D> Union<T, D>::from_iterator_range(const Iter &begin, const Iter &end)
{
	Union<T, D> shape;
	shape.init_from_iter_range(begin, end);
	return shape;
}

template<typename T, unsigned D>
inline Union<T, D> *Union<T, D>::clone() const
{
	return new Union<T, D>(*this);
}

} // namespace Geometry
} // namespace Msp

#endif
